;+
; NAME:
;	READ_APEX
;
; PURPOSE:
;	Restore or read MBFits data from APEX for a given subscan.
;
; CALLING SEQUENCE:
;	Datastr = READ_APEX(Scan_number, Subscan, Filetype, band=band, Goodpix_ima=Goodpix_ima, Calibration_camera=Calibration_camera [,slice_otf=slice_otf])
;
; INPUTS:
;	Scan_number:	Scan number.
;	Subscan:	Subscan number.
;	Goodpix_ima:	Good pixels array, 1 for good pixels, 0 for bad ones.
;	Calibration_camera:	Input calibration file.
;
; OUTPUTS
;	Filetype:	fits or xdr (format)       PG+PhA
;
; KEYWORD PARAMETERS:
;       BAND:           Spectral band (channel) 350, 450 or 200 microns. Default is 350 microns
;	NOPOWERMAP:	Do not calibrate from V to pW
;	SLICE_OTF:	Slice OTF data from a unique scan into subscans
;		
; COMMON BLOCKS:
;	OBS1_CONFIGB:	Contains working directory, calibration table and project name
;		
; EXAMPLE:
;		Donnees = READ_APEX(47527, 10, 'MAP', Goodpix_ima=Goodpix_ima, Calibration_camera=Calibration_camera, /NOPOWERMAP)
;
; MODIFICATION HISTORY:
;  - 2014     - P. Gallais  - created
;  - 2016     - P. Gallais  - 'band' argument added
;  - 2016     - Ph. Andre   - correction for time shifts
;  - 20170912 - F. Schuller - adapted to raw data stored in a single FITS file
;-

function read_apex, scan_number, subscan, filetype, band=band, calibration_camera=calibration_camera, $
                    goodpix_ima=goodpix_ima, nopowermap=nopowermap, slice_otf=slice_otf

COMMON obs1_configb

  if n_params() eq 0 then begin
	  print, 'Calling sequence :'
    print, '	READ_APEX, scan_number, subscan, filetype, band=band, calibration_camera=calibration_camera, goodpix_ima=goodpix_ima, nopowermap=nopowermap'
    return, -1
  endif

  if not keyword_set(band) then spectral_band = 350 else spectral_band = band

  apexdata = work_dir+'apexdata/'
  dirdef   = apexdata + 'basic_xdr/' + strtrim(spectral_band,2) + '/'
  filename = dirdef + '*' + '_' + strtrim(string(scan_number),2) + '_' + subscan + '.xdr'

  ; Look for existing .xdr file
  file = file_search(filename, count=nfile)
  datastr = -1
  if nfile eq 1 then begin
	  print, 'READ_APEX : restoring ' + file(0)
	  restore, file(0)
	  filetype='xdr'
  endif else begin   ; no .xdr file, read raw data from MB-FITS
	  print, 'READ_APEX : reading fits ' + strtrim(string(scan_number), 2) + '  ' + subscan 
    if keyword_set(nopowermap) then begin
      datastr = readfits_apex(scan_number, subscan, band=spectral_band, $
                              calibration_camera=calibration_camera, goodpix_ima=goodpix_ima, nopowermap=nopowermap)
    endif else begin
      datastr = readfits_apex(scan_number, subscan, band=spectral_band, $
                              calibration_camera=calibration_camera, goodpix_ima=goodpix_ima) 
    endelse
    if datatype(datastr) NE 'STC' then return, -1
    
    delta_longoff = max(datastr.datapar.longoff)-min(datastr.datapar.longoff)
    delta_latoff = max(datastr.datapar.latoff)-min(datastr.datapar.latoff)

    if (finite(delta_longoff) ne 1 or finite(delta_latoff) ne 1 or delta_longoff gt 1. or delta_latoff gt 1.) then begin
    	print, "delta_longoff (deg)  = ", delta_longoff, "   delta_latoff (deg) = ", delta_latoff,$
    	       "   Ignoring subscan ", datastr.subscan, " for scan ", datastr.scannum
    	return, -1
    endif

;PhA2016 Correcting for time shift at 350mu (~ 18 frames) for data taken in 2016 June
    if spectral_band eq 350 and strmid(datastr.date_obs,0,4) eq '2016' and datastr.scannum gt 29720 and datastr.scannum lt 34731 then begin
      if datatype(datastr.datapar) eq 'STC' then begin
	      ntime = (size(datastr.datapar))(1)
	      if ntime gt 18 then begin
	    	  datastr_save = datastr
	    	  datastr.datapar[0:ntime-1-18] = datastr_save.datapar[18:ntime-1]
	    	  datastr.datapar[ntime-18:ntime-1].longoff = -999.
	    	  datastr.datapar[ntime-18:ntime-1].latoff = -999.
	    	  datastr.lst[ntime-18:ntime-1] = -999.
	      endif	 
	    endif else begin
	      print, strcompress(string(subscan)), '  DATASTR.DATAPAR N EST PAS UNE STRUCTURE'
	      stop
	    endelse	 
    endif	
	
;PhA2016 Correcting for time shift at 450mu (~ -5 frames)
    if spectral_band eq 450 and strmid(datastr.date_obs,0,4) eq '2016' and datastr.scannum gt 29720 then begin 
      if datatype(datastr.datapar) eq 'STC' then begin
	      ntime = (size(datastr.datapar))(1)
	      if ntime gt 3 then begin
          datastr_save = datastr
    	    n_channel_x = 73      	    	    	    	    ; including 3*3 fake pixels between subarrays
    	    n_channel_y = 39      	    	    	    	    ; including 3 fake pixels between subarrays
          mask_jun16 = intarr(n_channel_x,n_channel_y)+1
          mask_jun16_2 = mask_jun16*0
          mask_jun16_2(19:34,0:17) = 1
;
;Correcting for time shift at 450mu (~ -3.4 frames)
    	    cube   = datastr.cube		
          cube_2 = datastr.cube
	    	  for kk=0L, ntime-1 do cube_2(*,*,kk) = cube_2(*,*,kk)*mask_jun16_2
		      cube_2_save = cube_2
	    	  cube_2(*,*,0:ntime-1-3) = cube_2_save(*,*,3:ntime-1)
          for kk=0L, ntime-1 do cube(*,*,kk) = cube(*,*,kk)*(mask_jun16-mask_jun16_2) + cube_2(*,*,kk)*mask_jun16_2    	    			
    	    datastr.cube = cube
        endif	 
      endif else begin
    	  print, strcompress(string(subscan)), '  DATASTR.DATAPAR N EST PAS UNE STRUCTURE'
	      stop
	    endelse
    endif 
;PhA2016
    
    ; check that datastr is a structure, and test for consistency of some parameters 
    check_data = check_data_partemis(datastr)
    ; continue only if check ok
    if check_data eq 1 then begin
      proj_name = datastr.PROJID
      ; do we need to slice the data?
      if keyword_set(slice_otf) then begin
        slice_otf, datastr, dirdef, proj_name, keep=slice_otf
        print, ' '
        print, 'SLICE_OTF : Slicing into subscans done, launch TRAITE_OTF_MAP_MAIN without /slice keyword'
		    return, -1
      endif else begin
        filename = dirdef + proj_name + '_' + strtrim(string(scan_number),2) + '_' + subscan + '.xdr'
        save, filename = filename, datastr, /xdr
        print, ' --------> data structure stored in: '+filename
      endelse
      filetype='fits'
    endif else begin
      ; datastr is NOT a structure
      print, 'Le subscan '+ subscan + ' contient des erreurs ', $
             '(cf Error_report_on_' + strtrim(string(scan_number),2) + '_data.txt) et a ete ignore'	
    endelse
  endelse
  return, datastr
end
